/*
 * Copyright 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <DemUnidraw/demclasses.h>
#include <DemUnidraw/demcmds.h>
#include <DemUnidraw/demcomps.h>
#include <DemUnidraw/demeditor.h>
#include <Attribute/paramlist.h>
#include <TopoFace/fgeomobjs.h>
#include <Unidraw/Graphic/graphic.h>
#include <Unidraw/iterator.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/viewer.h>
#ifndef MAKEDEPEND
#include <vtkCubeSource.h>
#include <vtkOpenGLActor.h>
#include <vtkPolyDataMapper.h>
#endif

/****************************************************************************/

ActorComp::ActorComp(const char* name, OverlayComp* parent) :
  OverlayComp(nil, parent) {
    _name = strdup(name);
    _xloc = _yloc = 0.0;
    _xdir = 0.0;
    _ydir = 1.0;
}

Component* ActorComp::Copy() {
  ActorComp* copy = new ActorComp(_name);
  return copy;
}

ClassId ActorComp::GetClassId() { return ACTOR_COMP; }

boolean ActorComp::IsA(ClassId id) {
  return id == ACTOR_COMP || OverlayComp::IsA(id);
}

ActorView::ActorView() : OverlayView() {
  _actor = nil;
}

ClassId ActorView::GetClassId() { return ACTOR_VIEW; }

boolean ActorView::IsA(ClassId id) {
  return id == ACTOR_VIEW || OverlayView::IsA(id);
}

Graphic* ActorView::GetGraphic() {
  if (!_gr)
    _gr = new FullGraphic();
  return _gr;
}

void ActorView::Update() {
  if (strcmp(((DemEditor*)GetViewer()->GetEditor())->VehicleName(),
	     ((ActorComp*)GetSubject())->Name()) == 0) {
    // camera update
    ActorComp* actor = (ActorComp*)GetSubject();
    float fx = actor->xloc() + actor->xdir()*5000.0;
    float fy = actor->yloc() + actor->ydir()*5000.0;
    DemCameraUpdateCmd* camcmd = new DemCameraUpdateCmd(nil,
							actor->xloc(),
							actor->yloc(),
							fx,
							fy,
							false);
    camcmd->SetEditor(GetViewer()->GetEditor());
    camcmd->Execute();
  }
  else {
    // actor update
    if (!_actor) {  // create actor first time
      _actor = vtkOpenGLActor::New();
      vtkCubeSource* cube = vtkCubeSource::New();
      cube->SetXLength(3.0);
      cube->SetYLength(4.0);
      cube->SetZLength(2.0);
      vtkPolyDataMapper* mapper = vtkPolyDataMapper::New();
      mapper->SetInput(cube->GetOutput());

    }
    // move actor

  }
}

/****************************************************************************/

ParamList* SurfaceComp::_surface_params = nil;

SurfaceComp::SurfaceComp(const char* path, OverlayComp* parent)
  : OverlayFileComp(parent) {
    SetPathName(path);
}

SurfaceComp::SurfaceComp(istream& in, OverlayComp* parent)
  /*: OverlayFileComp(in, parent)*/ {
    _pathname = nil;
    _valid = GetParamList()->read_args(in, this);
    SetParent(this, parent);
}

ClassId SurfaceComp::GetClassId() { return SURFACE_COMP; }

boolean SurfaceComp::IsA(ClassId id) { return id == SURFACE_COMP; }

Component* SurfaceComp::Copy() {
    SurfaceComp* newsc = new SurfaceComp(GetPathName());
    return newsc;
}

ParamList* SurfaceComp::GetParamList() {
    if (!_surface_params) 
	GrowParamList(_surface_params = new ParamList());
    return _surface_params;
}

void SurfaceComp::GrowParamList(ParamList* pl) {
    pl->add_param("pathname", ParamStruct::keyword, &SurfaceScript::ReadPathName,
		  this, this);
}

SurfaceView::SurfaceView() : OverlayFileView() {
}

ClassId SurfaceView::GetClassId() { return SURFACE_VIEW; }

boolean SurfaceView::IsA(ClassId id) { return id == SURFACE_VIEW; }

#if 0
void SurfaceView::SetSubject(Component* comp) {
  ComponentView::SetSubject(comp);
  DemLoadDemCmd* cmd = new DemLoadDemCmd(((SurfaceComp*)GetSubject())->GetPathName());
  Iterator it;
  unidraw->First(it);
  if (unidraw->GetEditor(it)) {
  cmd->SetEditor(unidraw->GetEditor(it));
  cmd->Execute();
  }
}

void SurfaceView::Update() {
  DemLoadDemCmd* cmd = new DemLoadDemCmd(((SurfaceComp*)GetSubject())->GetPathName());
  Iterator it;
  unidraw->First(it);
  if (unidraw->GetEditor(it)) {
  cmd->SetEditor(unidraw->GetEditor(it));
  cmd->Execute();
  }
}
#endif
void SurfaceView::Load() {
  DemLoadDemCmd* cmd = new DemLoadDemCmd(((SurfaceComp*)GetSubject())->GetPathName());
  Iterator it;
  unidraw->First(it);
  if (unidraw->GetEditor(it)) {
  cmd->SetEditor(unidraw->GetEditor(it));
  cmd->Execute();
  }
}

SurfaceScript::SurfaceScript(SurfaceComp* subj) : OverlayFileScript(subj) {
}

ClassId SurfaceScript::GetClassId() { return SURFACE_SCRIPT; }

boolean SurfaceScript::IsA(ClassId id) { return id == SURFACE_SCRIPT; }

int SurfaceScript::ReadPathName (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    OverlayFileComp* filecomp = (OverlayFileComp*)addr1;

    char pathname[BUFSIZ];
    if (ParamList::parse_pathname(in, pathname, BUFSIZ, filecomp->GetBaseDir()) != 0)
	return -1;

    /* check pathname for recursion */
    OverlayComp* parent = (OverlayComp*) filecomp->GetParent();
    while (parent != nil) {
	if (parent->GetPathName() && strcmp(parent->GetPathName(), pathname) == 0) {
	    cerr << "pathname recursion not allowed (" << pathname << ")\n";
	    return -1;
	}
	parent = (OverlayComp*) parent->GetParent();
    }

    filecomp->SetPathName(pathname);
    return 0;
}

/****************************************************************************/

ParamList* RoadsComp::_roads_params = nil;

RoadsComp::RoadsComp(const char* path, OverlayComp* parent)
  : OverlayFileComp(parent) {
    SetPathName(path);
}

RoadsComp::RoadsComp(istream& in, OverlayComp* parent)
  /*: OverlayFileComp(in, parent)*/ {
    _pathname = nil;
    _valid = GetParamList()->read_args(in, this);
    SetParent(this, parent);
}

ClassId RoadsComp::GetClassId() { return ROADS_COMP; }

boolean RoadsComp::IsA(ClassId id) { return id == ROADS_COMP; }

Component* RoadsComp::Copy() {
    RoadsComp* newsc = new RoadsComp(GetPathName());
    return newsc;
}

ParamList* RoadsComp::GetParamList() {
    if (!_roads_params) 
	GrowParamList(_roads_params = new ParamList());
    return _roads_params;
}

void RoadsComp::GrowParamList(ParamList* pl) {
    pl->add_param("pathname", ParamStruct::keyword, &RoadsScript::ReadPathName,
		  this, this);
}

RoadsView::RoadsView() : OverlayFileView() {
}

ClassId RoadsView::GetClassId() { return ROADS_VIEW; }

boolean RoadsView::IsA(ClassId id) { return id == ROADS_VIEW; }

#if 0
void RoadsView::SetSubject(Component* comp) {
  ComponentView::SetSubject(comp);
  DemLoadDemCmd* cmd = new DemLoadDemCmd(((RoadsComp*)GetSubject())->GetPathName());
  Iterator it;
  unidraw->First(it);
  if (unidraw->GetEditor(it)) {
  cmd->SetEditor(unidraw->GetEditor(it));
  cmd->Execute();
  }
}

void RoadsView::Update() {
  DemLoadDemCmd* cmd = new DemLoadDemCmd(((RoadsComp*)GetSubject())->GetPathName());
  Iterator it;
  unidraw->First(it);
  if (unidraw->GetEditor(it)) {
  cmd->SetEditor(unidraw->GetEditor(it));
  cmd->Execute();
  }
}
#endif
void RoadsView::Load() {
  DemLoadFCCmd* cmd = new DemLoadFCCmd(((RoadsComp*)GetSubject())->GetPathName());
  Iterator it;
  unidraw->First(it);
  if (unidraw->GetEditor(it)) {
  cmd->SetEditor(unidraw->GetEditor(it));
  cmd->Execute();
  }
}

RoadsScript::RoadsScript(RoadsComp* subj) : OverlayFileScript(subj) {
}

ClassId RoadsScript::GetClassId() { return ROADS_SCRIPT; }

boolean RoadsScript::IsA(ClassId id) { return id == ROADS_SCRIPT; }

int RoadsScript::ReadPathName (istream& in, void* addr1, void* addr2, void* addr3, void* addr4) {
    OverlayFileComp* filecomp = (OverlayFileComp*)addr1;

    char pathname[BUFSIZ];
    if (ParamList::parse_pathname(in, pathname, BUFSIZ, filecomp->GetBaseDir()) != 0)
	return -1;

    /* check pathname for recursion */
    OverlayComp* parent = (OverlayComp*) filecomp->GetParent();
    while (parent != nil) {
	if (parent->GetPathName() && strcmp(parent->GetPathName(), pathname) == 0) {
	    cerr << "pathname recursion not allowed (" << pathname << ")\n";
	    return -1;
	}
	parent = (OverlayComp*) parent->GetParent();
    }

    filecomp->SetPathName(pathname);
    return 0;
}

/****************************************************************************/

ParamList* DemIdrawComp::_dem_idraw_params = nil;

DemIdrawComp::DemIdrawComp (const char* pathname, OverlayComp* parent)
: MapIdrawComp(pathname, parent) { }

ClassId DemIdrawComp::GetClassId () { return DEMIDRAW_COMP; }

boolean DemIdrawComp::IsA (ClassId id) {
    return DEMIDRAW_COMP == id || MapIdrawComp::IsA(id);
}

DemIdrawComp::DemIdrawComp (istream& in, const char* pathname, OverlayComp* parent) /* : MapIdrawComp(pathname, parent) */{
    _valid = GetParamList()->read_args(in, this);
}

ParamList* DemIdrawComp::GetParamList() {
    if (!_map_idraw_params) 
	GrowParamList(_dem_idraw_params = new ParamList());
    return _dem_idraw_params;
}

void DemIdrawComp::GrowParamList(ParamList* pl) {
  pl->add_param("kids", ParamStruct::optional, &OverlaysScript::ReadChildren, this, this);
  pl->add_param("other", ParamStruct::other,        &OverlayScript::ReadOther,
	this, this);
}

Component* DemIdrawComp::Copy () {
    return new DemIdrawComp(nil, nil);
}

/****************************************************************************/

DemIdrawView::DemIdrawView (DemIdrawComp* subj) : MapIdrawView(subj) { }

ClassId DemIdrawView::GetClassId () { return DEMIDRAW_VIEW; }

boolean DemIdrawView::IsA (ClassId id) {
    return DEMIDRAW_VIEW == id || MapIdrawView::IsA(id);
}

/*****************************************************************************/

DemIdrawScript::DemIdrawScript (DemIdrawComp* subj) : MapIdrawScript(subj) {}

ClassId DemIdrawScript::GetClassId () { return DEMIDRAW_SCRIPT; }

boolean DemIdrawScript::IsA (ClassId id) { 
    return DEMIDRAW_SCRIPT == id || MapIdrawScript::IsA(id);
}
