/*
 * Copyright 1997 Vectaport Inc.
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapcov.h>
#include <Map/mapfeature.h>
#include <Map/mapfclass.h>

#include <iostream.h>
#include <string.h>

#include <IV-2_6/_enter.h>

implementTable(MapFeatureTable,int,MapFeature*)

/*****************************************************************************/

MapFeatureClass::MapFeatureClass (MapCoverage* cov, const char* name) : MapFiles(name, nil) {
    coverage(cov);
    _path = nil;
    _coverage = cov;
    _nfeats = -1;
    _features = nil;
    _hidden = nil;
    _tiled = false;
    junctionindex = lineindex = nil;
    jit = lit = nil;
}

MapFeatureClass::MapFeatureClass(int nf, MapFeature** flist, const char* name) : MapFiles(name, nil) {
    _path = nil;
    _coverage = nil;
    _nfeats = nf;
    _features = flist;
    _hidden = nil;
    _tiled = false;
    junctionindex = lineindex = nil;
    jit = lit = nil;
}

MapFeatureClass::MapFeatureClass(int nf) : MapFiles(nil, nil) {
    _path = nil;
    _coverage = nil;
    _nfeats = 0;
    if (nf <= 0)
      _size = 10;
    else
      _size = nf;
    _features = new MapFeature*[_size];
    _hidden = nil;
    _tiled = false;
    junctionindex = lineindex = nil;
    jit = lit = nil;
}

void MapFeatureClass::append(MapFeature* mf) {
  if (_nfeats == _size) {
    MapFeature** newarray = new MapFeature*[_size+10];
    for (int i = 0; i < _nfeats; i++)
      newarray[i] = _features[i];
    delete [] _features;
    _features = newarray;
  }
  _features[_nfeats++] = mf;
}

MapFeatureClass::~MapFeatureClass () {
    if (_features) {
	for (int i=0; i<_nfeats; i++) 
	    delete _features[i];
	delete [] _features;
    }
    delete junctionindex;
    delete lineindex;
    delete jit;
    delete lit;
}

void MapFeatureClass::make_junction_idx() {
  junctionindex = new MapFeatureTable(_nfeats);
  for (int i = 0; i < _nfeats; i++) {
    MapFeature* f = _features[i];
    if (f->type() == MapFeature::JunctionType)
      junctionindex->insert(f->id(), f);
  }
}

void MapFeatureClass::make_line_idx() {
  lineindex = new MapFeatureTable(_nfeats);
  for (int i = 0; i < _nfeats; i++) {
    MapFeature* f = _features[i];
    if (f->type() == MapFeature::LineType)
      lineindex->insert(f->id(), f);
  }
}

MapFeature* MapFeatureClass::junction_by_id(int id) {
  if (!junctionindex)
    make_junction_idx();
  MapFeature* jf = nil;
  junctionindex->find(jf, id);
  return jf;
}

MapFeature* MapFeatureClass::line_by_id(int id) {
  if (!lineindex)
    make_line_idx();
  MapFeature* lf = nil;
  lineindex->find(lf, id);
  return lf;
}

MapFeature* MapFeatureClass::first_junction() {
  if (!junctionindex)
    make_junction_idx();
  delete jit;
  jit = new TableIterator(MapFeatureTable)(*junctionindex);
  return jit->cur_entry() ? jit->cur_value() : nil;
}

MapFeature* MapFeatureClass::next_junction() {
  if (!jit)
    return first_junction();
  else {
    jit->next();
    return jit->cur_entry() ? jit->cur_value() : nil;
  }
}

MapFeature* MapFeatureClass::first_line() {
  if (!lineindex)
    make_line_idx();
  delete lit;
  lit = new TableIterator(MapFeatureTable)(*lineindex);
  return lit->cur_entry() ? lit->cur_value() : nil;
}

MapFeature* MapFeatureClass::next_line() {
  if (!lit)
    return first_line();
  else {
    lit->next();
    return lit->cur_entry() ? lit->cur_value() : nil;
  }
}

void MapFeatureClass::extent(double& x1, double& y1, double& x2, double& y2, int& utmzone) {
    x1 = _x1;
    y1 = _y1;
    x2 = _x2;
    y2 = _y2;
    utmzone = _zone;
}

const char* MapFeatureClass::path() const {
    if (_coverage && !_path) {
	const char* covpath = _coverage->path();
	const int len = strlen(covpath) + strlen(_name) + 2;
        ((MapFeatureClass*)this)->_path = new char[len];
	char* fclpath = ((MapFeatureClass*)this)->_path;
        strcpy( fclpath, covpath );
        strcat( fclpath, "/" );
        strcat( fclpath, _name );
    }
    return _path;
}

int MapFeatureClass::nfeats() const{
    if (!initialized()) ((MapFeatureClass*)this)->init_features();

    return _nfeats;
}

MapFeature* MapFeatureClass::feature(unsigned featnum) const{
    if (!initialized()) ((MapFeatureClass*)this)->init_features();
    
    if (_features && featnum<_nfeats)
	return _features[featnum];
    else
	return nil;
}
     
const char* MapFeatureClass::description() const { return "generic feature description"; }

ostream& operator<< (ostream& out, const MapFeatureClass& fclass) {
    out << "Feature Class:  " << fclass.name() << "\n";
    out << "Path:  " << fclass.path() << "\n";
    out << "Description:  " << fclass.description() << "\n";
    out << "Features:  " << fclass.nfeats() << "\n";
    return out;
}

void MapFeatureClass::init_features() {
    if (initialized()) return;

    _features = make_features(_nfeats);
}

boolean MapFeatureClass::tiled() {
    return _tiled;
}

void MapFeatureClass::tiled(boolean t) {
    _tiled = t;
}
